import * as THREE from 'https://unpkg.com/three@v0.150.0/build/three.module.js';

var renderer, scene, camera;
var chopper;
var textures = [];
var bumpTexture; // Store the bump texture in this variable
var texturesToLoad = 6; // Load a lot of textures
var textureIndex = 0;

var vertexShader, fragmentShader;

var lightPosition;
var lightTrajectory;

var viewerPosition = new THREE.Vector3(0.0, 2.0, 0.0);

function millis() {
    return (new Date()).getTime();
}

function toRad(degree) {
    return Math.PI * 2 * degree / 360;
}

window.onLoad = async () => {
    var canvasContainer = document.getElementById('myCanvasContainer');
    var width = 800;
    var height = 500;

    vertexShader = await (await fetch('vertex.glsl')).text();
    fragmentShader = await (await fetch('fragment.glsl')).text();

    renderer = new THREE.WebGLRenderer();
    renderer.setSize(width, height);
    canvasContainer.appendChild(renderer.domElement);

    scene = new THREE.Scene();

    camera = new THREE.PerspectiveCamera(80, width / height, 1, 1000);
    camera.position.set(viewerPosition.x, viewerPosition.y, viewerPosition.z);
    camera.up = new THREE.Vector3(0, 0, -1);
    camera.lookAt(new THREE.Vector3(0, 0, 0));
    scene.add(camera);

    lightTrajectory = new THREE.CatmullRomCurve3([
        new THREE.Vector3(-5, -7, 5),
        new THREE.Vector3(-4, -7, -2),
        new THREE.Vector3(3, -7, -4),
        new THREE.Vector3(8, -7, 6),
    ]);
    lightTrajectory.closed = true;

    window.addEventListener('keydown', function (event) {
        if (event.code == "ArrowRight") {
            changeTexture(-1);
        }
        if (event.code == "ArrowLeft") {
            changeTexture(+1);
        }
    }, false);

    scene.add(new THREE.Mesh(new THREE.SphereGeometry(0.1, 6, 6), new THREE.MeshBasicMaterial({ color: 0xeeeeee })));

    var loader = new THREE.TextureLoader();
    loader.setCrossOrigin('Anonymous');
    loader.setPath('http://cglearn.eu/files/course/7/textures/');

    // Here are a lot of textures you can try your bump mapping on.
    loader.load('sasuke256.png', onTextureLoaded);
    loader.load('ut256.png', onTextureLoaded);
    loader.load('lines.png', onTextureLoaded);
    loader.load('grid.png', onTextureLoaded);
    loader.load('white.png', onTextureLoaded);

    // This will load the bump map texture.
    // There are actually several of those you can try out: 
    //   bumpTexture.jpg - "asphalt" texture, lot of small bumps
    //   bumpTexture2.jpg - crumpled paper, few sharp bumps
    //   bumpTexture3.jpg - brick wall 
    //   bumpTexture.png - big white shape, use this for initial testing
    // More free textures: http://freeseamlesstextures.com/gallery/
    loader.load('bumpTexture.png', onBumpTextureLoaded);
    // loader.load('bumpTexture3.jpg', onBumpTextureLoaded);
}

/**
 * --Task-- This function is called when the bump texture is loaded.
 * Finish this function.
 */
function onBumpTextureLoaded(texture) {
    // --Task-- Configure the bump texture.
    // Set it to the global variable "bumpTexture"

    if (--texturesToLoad == 0) {
        onAllTexturesLoaded();
    }
}

/**
 * --Task-- This function is called when one texture is loaded.
 * Finish this function (copy from the previous task).
 */
function onTextureLoaded(texture) {
    // --Task-- Configure the other textures
    // Copy what you did in the previous task.

    if (--texturesToLoad == 0) {
        onAllTexturesLoaded();
    }
}

function onAllTexturesLoaded() {
    addQuad(textures[textureIndex]);
    draw();
}

/**
 * This function changes the texture and updates it in the material.
 */
function changeTexture(addend) {
    textureIndex = (textureIndex + addend + 5) % 5
    scene.children[2].material.uniforms.colorTexture.value = textures[textureIndex];
    scene.children[2].material.needsUpdate = true;
    console.log(textureIndex);
}

function draw() {
    requestAnimationFrame(draw);

    var m = millis() / 4000;
    lightPosition = lightTrajectory.getPoint(m - parseInt(m));
    scene.children[1].position.set(lightPosition.x, lightPosition.y, lightPosition.z);

    scene.traverse(function (object) {
        if (object.material !== undefined && object.material instanceof THREE.ShaderMaterial) {
            object.material.uniforms.lightPosition.value = new THREE.Vector3().copy(lightPosition).applyMatrix4(camera.matrixWorldInverse);
        }
    });

    rotatePlane();

    renderer.render(scene, camera);
}

/**
 * --Task-- 
 * Make the plane rotate in the range [-15, 15] degrees around the y axis
 * Use a cosine function for interpolation.
 */
function rotatePlane() {
    // --Task-- Calculate the rotation and change the y component here
    scene.children[2].needsUpdate = true; // Afterwards we update the mesh
}

function addQuad(texture) {
    var halfPi = Math.PI / 2;

    var quad = createQuad(texture);
    quad.position.set(0, -10, 0);
    quad.rotation.set(-halfPi, 0, 0);

    scene.add(quad);
}

function createQuad(texture) {
    var geometry = new THREE.PlaneGeometry(20, 20, 1);
    var material = createShaderMaterial(texture);
    var wall = new THREE.Mesh(geometry, material);

    return wall;
}

function createShaderMaterial(texture) {

    return new THREE.ShaderMaterial({
        // This time we'll use three.js built-in UV mapping in the shader.
        // It is the same mapping you created in the previous task. :)
        uniforms: {
            lightPosition: {
                type: 'v3',
                value: lightPosition
            },
            colorTexture: {
                type: 't',
                value: texture
            },
            bumpTexture: {
                type: 't',
                value: bumpTexture
            }
        },
        vertexShader: vertexShader,
        fragmentShader: fragmentShader
    });
}